import pandas as pd

from datetime import datetime, time
from dotenv import load_dotenv
from os import getenv
from sqlalchemy import create_engine

load_dotenv()

database_url = (
    f"postgresql+psycopg2://{getenv('DATABASE_USER')}:{getenv('DATABASE_PASSWORD')}"
    f"@{getenv('DATABASE_HOST')}:{getenv('DATABASE_PORT')}/{getenv('DATABASE_NAME')}"
)

def trade(date: datetime, symbol: str, strategy: str, entry_time: datetime) -> pd.DataFrame:
    """Retrieve a specific trade record."""
    query = f"""
        SELECT *
        FROM "Trades"
        WHERE "Date" = %s
          AND "Symbol" = %s
          AND "Strategy" = %s
          AND "Entry Time" = %s
    """
    params = (datetime.combine(date, time()), symbol, strategy, entry_time)

    return pd.read_sql_query(query, create_engine(database_url), params = params)

def trades(date: datetime = None) -> pd.DataFrame:
    """Retrieve trades based on a specified date."""
    query = f"""
        SELECT *
        FROM "Trades"
        WHERE "Date" = %s
        ORDER BY "Entry Time";
    """
    params = (date, )

    return pd.read_sql_query(query, create_engine(database_url), params = params)