import json
import pandas as pd

from dotenv import load_dotenv
from os import getenv
from sqlalchemy import create_engine, text

load_dotenv()

def upsert(trade_data: pd.DataFrame):
    trade_data['Spreads'] = trade_data['Spreads'].apply(lambda x: json.dumps(x) if isinstance(x, list) else x)

    database_url = (
        f"postgresql+psycopg2://{getenv('DATABASE_USER')}:{getenv('DATABASE_PASSWORD')}"
        f"@{getenv('DATABASE_HOST')}:{getenv('DATABASE_PORT')}/{getenv('DATABASE_NAME')}"
    )
    engine = create_engine(database_url)

    upsert_statement = text("""
        INSERT INTO "Trades" ("Date", "Symbol", "Strategy", "Entry Time", "Exit Time", "Spreads", "Profit")
        VALUES (:Date, :Symbol, :Strategy, :Entry_Time, :Exit_Time, :Spreads, :Profit)
        ON CONFLICT ("Date", "Symbol", "Strategy", "Entry Time") DO UPDATE 
        SET "Exit Time" = EXCLUDED."Exit Time",
            "Spreads" = EXCLUDED."Spreads",
            "Profit" = EXCLUDED."Profit"
    """)

    # Using engine.begin() here as engine.connect() results in a rollback even though there is no error.
    # Still not sure why this is happening, but the following works:
    with engine.begin() as conn:
        for _, row in trade_data.iterrows():
            row = row.rename({
                'Entry Time': 'Entry_Time',
                'Exit Time': 'Exit_Time'
            })
            conn.execute(upsert_statement, row.to_dict())